"""
gvcf2bed tests
~~~~~~~~~~~~~~~~

:copyright: (c) 2016 Sander Bollen
:copyright: (c) 2016 Leiden University Medical Center
:license: MIT
"""
import vcf
import pytest
from gvcf2bed import *


@pytest.fixture()
def mini_reader():
    reader = vcf.Reader(filename="test/data/mini.vcf")
    return reader


@pytest.fixture()
def block_reader():
    reader = vcf.Reader(filename="test/data/block.vcf")
    return reader


class TestFunctions(object):

    def test_gqx_mini(self, mini_reader):
        gqx = [get_gqx(x, "Sample_01") for x in mini_reader]
        assert len(gqx) == 11
        assert gqx == [20.0, 21.0, 23.0, 24.0, 25.0, 26.0, 27.0, 28.0, 29.0, 30.0, 31.0]

    def test_gqx_block(self, block_reader):
        gqx = [get_gqx(x, "Sample_01") for x in block_reader]
        assert len(gqx) == 10
        assert gqx == [3.0, 5.0, 3.0, 2.0, 3.0, 2.0, 3.0, 2.0, 3.0, 2.0]

    def test_bedline(self):
        tmp = BedLine("chr1", 500, 600)
        assert str(tmp) == "chr1\t500\t600"

    def test_vcf2bed(self, mini_reader):
        records = [vcf_record_to_bed(x) for x in mini_reader]
        assert len(records) == 11
        assert str(records[0]) == "chr1\t156084708\t156084709"

    def test_vcf2bed_block(self, block_reader):
        records = [vcf_record_to_bed(x) for x in block_reader]
        assert len(records) == 10
        assert str(records[0]) == "chr1\t83880332\t83880428"








"""
gvcf2bed tool
~~~~~~~~~~~~~


:copyright: (c) 2016 Sander Bollen
:copyright: (c) 2016 Leiden University Medical Center
:license: MIT
"""

import argparse
from collections import namedtuple
import vcf


class BedLine(namedtuple("BedLine", ["chromosome", "start", "end"])):

    def __str__(self):
        return "{0}\t{1}\t{2}".format(self.chromosome, self.start, self.end)


def get_gqx(record, sample):
    """
    Get GQX value from a pyvcf record
    :param record: an instance of a pyvcf Record
    :param sample: sample name
    :return: float
    """
    fmt = record.genotype(sample)
    if hasattr(fmt.data, "GQ") and record.QUAL:
        return min(float(fmt.data.GQ), record.QUAL)
    elif hasattr(fmt.data, "GQ"):
        return float(fmt.data.GQ)
    elif record.QUAL:
        return record.QUAL
    else:
        return 0.0


def vcf_record_to_bed(record):
    """
    Convert a VCF record to a BED record
    :param record: vcf record
    :return: BedLine record
    """
    if "END" in record.INFO:
        return BedLine(record.CHROM, record.start, record.INFO['END'])
    return BedLine(record.CHROM, record.start, record.end)


def main():
    desc = """
    Create a BED file from a gVCF.
    Regions are based on a minimum genotype quality.
    The gVCF file must contain a GQ field in its FORMAT fields.
    """
    parser = argparse.ArgumentParser(description=desc)

    parser.add_argument("-I", "--input", type=str, required=True, help="Input gVCF")
    parser.add_argument("-O", "--output", type=str, required=True, help="Output bed file")
    parser.add_argument("-s", "--sample", type=str, required=False, help="Sample name in VCF file to use. "
                                                                         "Will default to first sample "
                                                                         "(alphabetically) if not supplied")
    parser.add_argument("-q", "--quality", type=int, default=20, help="Minimum genotype quality (default 20)")

    args = parser.parse_args()

    reader = vcf.Reader(filename=args.input)
    if not args.sample:
        args.sample = sorted(reader.samples)[0]

    with open(args.output, "w") as ohandle:
        for record in reader:
            if get_gqx(record, args.sample) >= args.quality:
                ohandle.write(str(vcf_record_to_bed(record)) + "\n")

if __name__ == "__main__":
    main()



"""
setup.py
~~~~~~~~~~~~~


:copyright: (c) 2016 Sander Bollen
:copyright: (c) 2016 Leiden University Medical Center
:license: MIT
"""
from os.path import abspath, dirname, join

from setuptools import setup

readme_file = join(abspath(dirname(__file__)), "README.md")
with open(readme_file) as desc_handle:
    long_desc = desc_handle.read()

setup(
    name="gvcf2bed",
    version="0.1.1",
    description="Convert gVCF into BED",
    long_description=long_desc,
    author="Sander Bollen",
    author_email="a.h.b.bollen@lumc.nl",
    url="https://github.com/sndrtj/gvcf2bed",
    license="MIT",
    packages=["gvcf2bed"],
    install_requires=["pyvcf==0.6.8"],
    test_requires=["pytest", "pytest-cov"],
    entry_points={
        "console_scripts": [
            "gvcf2bed = gvcf2bed.gvcf2bed:main"
        ]
    },
    classifiers=[
        "Topic :: Scientific/Engineering :: Bio-Informatics"
    ]

)
